#!/usr/bin/python

import sys
import json
import sqlite3

class DbContext:
   def __init__(self, filename):
      self.dbconn = sqlite3.connect(filename)

   def __enter__(self):
      return self

   def __exit__(self, type, value, traceback):
      if type:
         print("Error occured while handling data, Rolling back any changes to the database:\n '{0}':'{1}'\n{2}".format(type, value, traceback))
         self.dbconn.rollback()
      else:
         self.dbconn.commit()
      self.dbconn.close()
      return True
   
   def addBatch(self, jsondict, table):
      for row in jsondict.get(table, []):
         self.addIng(row, table)

   def addIng(self, x, tableName):
      cursor = self.dbconn.cursor()
      setClause = ''
      for c, v in x.items():
         setClause = setClause + '{0} = "{1}", '.format(c,v)
      setClause = setClause[:-2]
      cursor.execute('INSERT INTO {0} DEFAULT VALUES'.format(tableName))
      newId = cursor.lastrowid
      cursor.execute('UPDATE {0} SET {1} WHERE id = {2}'.format(tableName, setClause, newId))
      cursor.execute('INSERT INTO bt_{0} DEFAULT VALUES'.format(tableName))
      newBtId = cursor.lastrowid
      cursor.execute(
         'UPDATE bt_{0} '.format(tableName) +
         'SET {0}_id = '.format(tableName) + str(newId) + ' ' +
         'WHERE id = ' + str(newBtId)
         )

if __name__ == "__main__" :
   jsonFilename = sys.argv[1]
   databaseFilename = '../data/default_db.sqlite'
   with open(jsonFilename) as jsonFile, DbContext(databaseFilename) as c:
      bigDict = json.load(jsonFile)
      c.addBatch(bigDict, 'fermentable')
      c.addBatch(bigDict, 'hop')
      c.addBatch(bigDict, 'misc')
      c.addBatch(bigDict, 'yeast')



#-----------------------------------------------------------------------------------------------------------------------
# scripts/btUtils.py is part of Brewtarget, and is copyright the following authors 2022-2024:
#   • Matt Young <mfsy@yahoo.com>
#
# Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.
#-----------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------
# This file contains various util functions used in our other Python build scripts
#-----------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------
# Python built-in modules we use
#-----------------------------------------------------------------------------------------------------------------------
import logging
import os
import pathlib
import subprocess

#-----------------------------------------------------------------------------------------------------------------------
# Helper function to return the 'base' directory (ie the one above the directory in which this file lives).
#-----------------------------------------------------------------------------------------------------------------------
def getBaseDir():
   dir_thisScript = pathlib.Path(__file__).parent.resolve()
   dir_base = dir_thisScript.parent.resolve()
   return dir_base

#-----------------------------------------------------------------------------------------------------------------------
# Helper function to return a logger that logs to stderr
#
# param logLevel - Initial level to log at
#-----------------------------------------------------------------------------------------------------------------------
def getLogger(logLevel = logging.INFO):
   logging.basicConfig(format='%(message)s')
   # Per https://docs.python.org/3/library/logging.html __name__ is the module’s name in the Python package namespace.
   # This is fine for us.  I don't think we care too much what the logger's name is.
   log = logging.getLogger(__name__)
   log.setLevel(logLevel)
   # Include the log level in the message
   handler = logging.StreamHandler()
   handler.setFormatter(
      # You can add timestamps etc to logs, but that's overkill for this script.  Source file location of log message is
      # however pretty useful for debugging.
      logging.Formatter('{levelname:s}:  {message}  [{filename:s}:{lineno:d}]', style='{')
   )
   log.addHandler(handler)
   # If we don't do this, everything gets printed twice
   log.propagate = False
   return log

#-----------------------------------------------------------------------------------------------------------------------
# Helper function for checking result of running external commands
#
# Given a CompletedProcess object returned from subprocess.run(), this checks the return code and, if it is non-zero
# stops this script with an error message and the same return code.  Otherwise the CompletedProcess object is returned
# to the caller (to make it easier to chain things together).
#-----------------------------------------------------------------------------------------------------------------------
def abortOnRunFail(runResult: subprocess.CompletedProcess):
   if (runResult.returncode != 0):
      # Per https://docs.python.org/3/library/logging.html, "Multiple calls to logging.getLogger() with the same name
      # [parameter] will always return a reference to the same Logger object."  So we are safe to set log in this way.
      log = logging.getLogger(__name__)

      # According to https://docs.python.org/3/library/subprocess.html#subprocess.CompletedProcess,
      # CompletedProcess.args (the arguments used to launch the process) "may be a list or a string", but its not clear
      # when it would be one or the other.
      if (isinstance(runResult.args, str)):
         log.critical('Error running ' + runResult.args)
      else:
         commandName = os.path.basename(runResult.args[0])
         log.critical('Error running ' + commandName + ' (' + ' '.join(str(ii) for ii in runResult.args) + ')')
      if runResult.stderr:
         log.critical('stderr: ' + runResult.stderr.decode('UTF-8'))
      exit(runResult.returncode)

   return runResult



#!/usr/bin/env python3
#-----------------------------------------------------------------------------------------------------------------------
# scripts/btInit.py is part of Brewtarget, and is copyright the following authors 2022-2024:
#   • Matt Young <mfsy@yahoo.com>
#
# Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.
#-----------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------
# This Python script is intended to be invoked by the `bt` bash script in the parent directory.  See comments in that
# script for why.
#
# Here we do some general set-up, including ensuring that a suitable Python virtual environment (venv) exists, before
# returning to the `bt` script to switch into that environment and invoke the main Python build tool script
# (buildTool.py)
#-----------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------
# Python built-in modules we use
#-----------------------------------------------------------------------------------------------------------------------
import os
import platform
import shutil
import subprocess
import sys

#-----------------------------------------------------------------------------------------------------------------------
# Our own modules
#-----------------------------------------------------------------------------------------------------------------------
import btUtils

log = btUtils.getLogger()


#
# First step is to ensure we have the minimum-required Python packages, including pip, are installed at system level.
#
# Prior to Python 3.12, setuptools is required to create and use virtual environments.
#
match platform.system():
   case 'Linux':
      # We don't want to run a sudo command every time the script is invoked, so check whether it's necessary
      exe_pip = shutil.which('pip3')
      ranUpdate = False
      if (exe_pip is None or exe_pip == ''):
         log.info('Need to install pip')
         btUtils.abortOnRunFail(subprocess.run(['sudo', 'apt', 'update']))
         ranUpdate = True
         btUtils.abortOnRunFail(subprocess.run(['sudo', 'apt', 'install', 'python3-pip']))
      # This is a bit clunky, but it's the simplest way to see if setuptools is already installed.  (Alternatively we
      # could run `pip3 list` and search for setuptools in the outpout.)
      foundSetupTools = False
      try:
         import setuptools
      except ImportError:
         log.info('Need to install setuptools')
      else:
         foundSetupTools = True
      if (not foundSetupTools):
         if (not ranUpdate):
            btUtils.abortOnRunFail(subprocess.run(['sudo', 'apt', 'update']))
            ranUpdate = True
         btUtils.abortOnRunFail(subprocess.run(['sudo', 'apt', 'install', 'python3-setuptools']))
      # It's a similar process for ensurepip, which is needed by venv below
      foundEnsurepip = False
      try:
         import ensurepip
      except ImportError:
         log.info('Need to install ensurepip')
      else:
         log.info('Found ensurepip')
         foundEnsurepip = True
      if (not foundEnsurepip):
         if (not ranUpdate):
            btUtils.abortOnRunFail(subprocess.run(['sudo', 'apt', 'update']))
            ranUpdate = True
         # Yes, I know it's confusing that we have to install a package called venv to ensure that the venv command
         # below doesn't complain about ensurepip not being present.  We're just doing what the error messages tell us
         # to.
         btUtils.abortOnRunFail(subprocess.run(['sudo', 'apt', 'install', 'python3-venv']))
   case 'Windows':
      #
      # In the past, we were able to install pip via Python, with the following code:
      #
      #    # https://docs.python.org/3/library/sys.html#sys.executable says sys.executable is '"the absolute path of the
      #    # executable binary for the Python interpreter, on systems where this makes sense".
      #    log.info(
      #       'Attempting to ensure latest version of pip is installed via  ' + sys.executable + ' -m ensurepip --upgrade'
      #    )
      #    btUtils.abortOnRunFail(subprocess.run([sys.executable, '-m', 'ensurepip', '--upgrade']))
      #
      # However, as of 2024-11, this gives "error: externally-managed-environment" and a direction "To install Python
      # packages system-wide, try 'pacman -S $MINGW_PACKAGE_PREFIX-python-xyz', where xyz is the package you are trying
      # to install."  So now we do that instead.  (Note that MINGW_PACKAGE_PREFIX will normally be set to
      # "mingw-w64-x86_64".)  As in buildTool.py, we need to specify '--overwrite' options otherwise we'll get "error:
      # failed to commit transaction (conflicting files)".
      #
      log.info('Install pip (' + os.environ['MINGW_PACKAGE_PREFIX'] + '-python-pip) via pacman')
      btUtils.abortOnRunFail(
         subprocess.run(['pacman', '-S',
                         '--noconfirm',
                         '--overwrite', '*python*',
                         '--overwrite', '*pip*',
                         os.environ['MINGW_PACKAGE_PREFIX'] + '-python-pip'])
      )
      #
      # Similarly, in the past, we were able to install setuptools as follows:
      #
      #    # See comment in scripts/buildTool.py about why we have to run pip via Python rather than just invoking pip
      #    # directly eg via `shutil.which('pip3')`.
      #    log.info('python -m pip install setuptools')
      #    btUtils.abortOnRunFail(subprocess.run([sys.executable, '-m', 'pip', 'install', 'setuptools']))
      #
      # But, as of 2024-11, this gives an error "No module named pip.__main__; 'pip' is a package and cannot be directly
      # executed".  So now we install via pacman instead.
      #
      log.info('Install setuptools (' + os.environ['MINGW_PACKAGE_PREFIX'] + '-python-setuptools) via pacman')
      btUtils.abortOnRunFail(
         subprocess.run(['pacman', '-S',
                         '--noconfirm',
#                         '--overwrite', '*python*',
#                         '--overwrite', '*pip*',
                         os.environ['MINGW_PACKAGE_PREFIX'] + '-python-setuptools'])
      )
   case 'Darwin':
      # Assuming it was Homebrew that installed Python, then, according to https://docs.brew.sh/Homebrew-and-Python,
      # it bundles various packages, including pip.  Since Python version 3.12, Homebrew marks itself as package manager
      # for the Python packages it bundles, so it's an error to try to install or update them via Python.
      log.info('Assuming pip is already up-to-date; installing python-setuptools')
      btUtils.abortOnRunFail(subprocess.run(['brew', 'install', 'python-setuptools']))

   case _:
      log.critical('Unrecognised platform: ' + platform.system())
      exit(1)

exe_python = shutil.which('python3')
log.info('sys.version: ' + sys.version + '; exe_python: ' + exe_python + '; ' + sys.executable)

#
# At this point we should have enough installed to set up a virtual environment.  In principle, it doesn't matter if the
# virtual environment already exists, as we are only using it to run the scripts/buildTool.py script.  In practice, life
# is a lot easier if we always start with a new virtual environment.  Partly this is because it makes debugging the
# scripts easier.  But more importantly, if there are old versions of Python sitting around in a previously-used venv,
# then some the paths won't get set up correctly, and we won't be able to find modules we install in the venv.  There
# will be multiple site-packages directories (one for each version of Python in the venv) but none of them will be in
# the search path for packages.
#
# Fortunately, venv can clear any existing environment for us with the '--clear' parameter
#
# Once running inside the virtual environment, any packages we need there can be installed directly in the venv with
# Python and Pip.
#
dir_venv = btUtils.getBaseDir().joinpath('.venv')
log.info('Create new Python virtual environment in ' + dir_venv.as_posix())
btUtils.abortOnRunFail(
   subprocess.run([sys.executable, '-m', 'venv', '--clear', dir_venv.as_posix()])
)

# Control now returns to the bt bash script in the parent directory
